from test_junkie.cli.cli import Cli


def main():

    Cli()


"""
0. support test coverage!
1. Runner to take config path
2. .resources to save in designated directory or to specific file
3. limits to reference the config if config was set
4. cancel to return when suites are canceled
5. save html & xml to a designated directory or specific file
6. test cov for the new modules
7. Tests needs to be filtered on string or object!!!
"""
